/*
 * Copyright (c) 2023-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at:
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef DOCA_ACTION_SPLIT_H
#define DOCA_ACTION_SPLIT_H

#include <doca_flow.h>

#include "dpdk-offload-provider.h"
#include "ovs-doca.h"

typedef int (*doca_offload_create_cb)(struct netdev *netdev,
                                      const struct rte_flow_attr *attr,
                                      struct rte_flow_item *items,
                                      struct rte_flow_action *actions,
                                      struct dpdk_offload_handle *doh,
                                      struct rte_flow_error *error);
typedef int (*doca_offload_destroy_cb)(struct netdev *netdev, struct dpdk_offload_handle *doh,
                                       struct rte_flow_error *error);

int
doca_action_split_create(doca_offload_create_cb offload_cb,
                         doca_offload_destroy_cb destroy_cb,
                         struct netdev *netdev,
                         const struct rte_flow_attr *attr,
                         struct rte_flow_item *items,
                         struct rte_flow_action *actions,
                         struct dpdk_offload_handle *doh,
                         struct rte_flow_error *error);
int
doca_action_split_destroy(doca_offload_destroy_cb destroy_cb,
                          struct netdev *netdev,
                          struct dpdk_offload_handle *doh,
                          struct rte_flow_error *error);

#endif /* DOCA_ACTION_SPLIT_H */
